{
    Project: Asteroids
    Company: Word in Action
             Copyright (C) 1995 by Jay Giganti.  All Rights Reserved.

    File    : Asters.pas         
    Author  : Jay Giganti
    Overview: This program makes up the delphi Code for an steroids type game
              that is based on a program from the book "Tricks of the game
              programming Gurus". by Lamothe, Ratcliff, Seminatore & Tyler.

              This is a simple game that I played with so I could start
              learning about game programming. There are still a few minor
              issues that could be fixed and some enhancements to the game
              but I though I would throw it out there to anyone interested.

              I do not sit in a tight loop and process the game which
              could easily be done when the timer first fires it then just
              disable the timer, and loop until the person has won,
              or been blown up while calling Application.ProcessMessages at
              the end of the loop. The reason is the game moves to fast if
              I do.  Currently the timer gives a resolution of 18.2 times a
              second and that is fast enough for now.

              Currently I can not seem to trap wmKeyDown messages so you have
              to keep hitting the keys to move. Also once you select A button
              the focus stays on them and the arrows will select different
              buttons and the space bar presses it down but since I eat the
              keys in the KeyUp area nothing happens. It is just visually
              annoying.

              You can use this code however you wish, it is released into
              the public domain. You need to mention the book above and
              me if you use it for commercial purposes.

              Any comments or suggestions or improvements to the game:
                  Jay Giganti   CompuServe ID: [72037,225]

                              Version Information

      Date    Ver  Comments
    ======== ===== ===========================================================
    05/13/95  1.00 Began work on this program
}


unit Asters;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, IniFiles, MPlayer, AboutBox;

type
  TForm1 = class(TForm)
    Timer1: TTimer;
    SndPlayer: TMediaPlayer;
    Panel1: TPanel;
    btnPause: TButton;
    Panel3: TPanel;
    clsButton: TButton;
    btnReplay: TButton;
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure clsButtonClick(Sender: TObject);
    procedure btnPauseClick(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btnReplayClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure BeginGame;
    procedure PlaySound(nSound: Integer);
    procedure UpdateCaption;
  end;

var
  Form1: TForm1;

implementation
{$R *.DFM}

type

TVertex = object
  x: Real;
  y: Real;
end;

TGraphObject = class
  nVerts   : Integer;
  Color    : TColor;
  nW       : Integer;
  nH       : Integer;
  xo       : Real;
  yo       : Real;
  Angle    : Real;
  xVeloc   : Real;
  yVeloc   : Real;
  cs       : Real;
  sn       : Real;
  Canvas   : TCanvas;
  rcBound  : TRect;
  Vertices : Array[0..14] of TVertex;

  constructor Create(Canv: TCanvas; Width, Height: Integer); dynamic;
  procedure   Free; dynamic;
  function    Collide(tgO: TGraphObject): Boolean;
  procedure   Draw(Erase: Boolean);
  procedure   MoveIt;
  procedure   Rotate;
  procedure   SetColor(Clr: TColor);
  procedure   SetPos(xPos, yPos: Real);
  procedure   SetRotate(NewAngle: Real);
  procedure   SetScale(Scale: Real);
  procedure   SetVertices(Verts: Array of TVertex);
  procedure   SetVelocity(xV, yV: Real);
  procedure   Translate;
end;

TDirectional = class(TGraphObject)
  nSpeed    : Integer;
  nMaxSpeed : Integer;
  nFacing   : Integer;
  csFace    : Real;
  snFace    : Real;

  constructor Create(Canv: TCanvas; Width, Height, MaxSpeed: Integer); dynamic;
  procedure   DecSpeed;
  procedure   IncSpeed;
  procedure   MoveIt; dynamic;
  procedure   Reset; dynamic;
  procedure   SetFacing(F: Integer);
  procedure   SetSpeed(S: Integer);
  procedure   UpdateVelocity;
  procedure   UpdateFacing(F: Integer);
end;

TAsteroid = class(TGraphObject)
  bDestroyed : Boolean;

  constructor Create(Canv: TCanvas; Width: Integer; Height: Integer);override;
  procedure   Destroy;
  procedure   MoveIt;
  procedure   Reset;
end;

TMissle = class(TDirectional)
  nAlive : Integer;

  constructor Create(Canv: TCanvas; Width, Height, MaxSpeed: Integer);override;
  procedure   MoveIt;
  procedure   DestoryAsters;
  procedure   SetAlive(nA: Integer);
end;

TShip = class(TDirectional)
  bRotate: Boolean;
  Missle : Array [0..3] of TMissle;

  constructor Create(Canv: TCanvas; Width, Height, MaxSpeed: Integer);override;
  procedure   Free; override;
  procedure   BlowUp;
  procedure   MoveIt;
  procedure   RotateLeft;
  procedure   RotateRight;
  procedure   Shoot;
  procedure   Reset;
end;

const
  NumMissles = 3;
  NumColors  = 13;
  MaxAsters  = 24;
  Margins    = 40;
  SndFire    = 1;
  SndHit     = 2;
  SndExplode = 3;
  SndWin     = 4;

var
  Asteroids  : Array[0..MaxAsters-1] of TAsteroid;
  Colors     : Array[0..NumColors-1] of TColor;
  Ship       : TShip;
  TimesDrawn : LongInt;
  StartTicks : LongInt;
  nScore     : Integer;
  FaceMod    : Real;
  NumDegrees : Integer;
  North      : Integer;
  NumAsters  : Integer;

{==============================================================================
                                 Vertex
Purpose: To convert a set of x,y corrdinates into a vertex stucture
==============================================================================}
function Vertex(xA, yA: Real): TVertex;
begin
  Result.x := xA;
  Result.y := yA;
end;

{------------------------------------------------------------------------------
                          TGraphObject Class Definitions
------------------------------------------------------------------------------}

{==============================================================================
                                 Create
Purpose: Construct and allocate an TGraphObject
Details: This will call the inherited create member for an TObject and then
         Initialized all of the internal Variables. This does not produce
         Much of an Object due to the fact that no vertices have been defined
         so each derived class needs to override this to define their shapes
==============================================================================}
constructor TGraphObject.Create(Canv: TCanvas; Width, Height: Integer);
begin
  inherited Create;

  Canvas  := Canv;
  nW      := Width;
  nH      := Height;
  xo      := 0;
  yo      := 0;
  Color   := clBlack;
  cs      := 0;
  sn      := 0;
  Angle   := 0;
  nVerts  := 0;
  xVeloc  := 0;
  yVeloc  := 0;
end;
{==============================================================================
                                 Free
Purpose: To release any resources of this object
Details: Cleanup Erases the object first then calls TObject free
==============================================================================}
procedure TGraphObject.Free;
begin
  Draw(TRUE);
  inherited Free;
end;

{==============================================================================
                                 Collide
Purpose: To determine if the current object collides with the passed in object
Details: The bounding rectangles are calcualted for the current object
         and the object passed in then if they intersect then you know
         that some part of them occupy the same space. The bounding rectangle
         is just the size need to encompas the object so you moust determine
         where the object is currenlty located and add that to the bounding
         rectangle

         This is crude hit detection due to the fact that bounding boxes
         are used and this might not be an exact hit due to the shape of
         the object It would be better to take the rcBox variable and then
         determine if the object really extends into this section.
==============================================================================}
function TGraphObject.Collide(tgO: TGraphObject): Boolean;
var
  rcBox  : TRect;
  rcCurOb: TRect;
  rcInObj: TRect;
begin

  Result  := False;
  rcCurOb := Rect(rcBound.Top    + TRUNC(xo),
                  rcBound.Left   + Trunc(yo),
                  rcBound.Bottom + TRUNC(xo),
                  rcBound.Right  + Trunc(yo));

  rcInObj := Rect(tgo.rcBound.Top    + TRUNC(tgo.xo),
                  tgo.rcBound.Left   + Trunc(tgo.yo),
                  tgo.rcBound.Bottom + TRUNC(tgo.xo),
                  tgo.rcBound.Right  + Trunc(tgo.yo));

  if (IntersectRect(rcBox, rcCurOb, rcInObj) <> 0) then
    Result := TRUE;
end;
{==============================================================================
                                 Draw
Purpose: To display the object at its current position on the screen
Details: If you are erasing then draw the object with the current forms color
         Then move to the first vertices, draw lines to all vertices, and then
         draw a line back to the starting vertice.
==============================================================================}
procedure TGraphObject.Draw(Erase: Boolean);
var
  nV: Integer;
begin
  with Canvas do
  begin
    if Erase = True then
      Pen.Color := Form1.Color
    else
      Pen.Color := Color;

    MoveTo(Trunc(xo + Vertices[0].x) ,Trunc(yo + Vertices[0].y));
    for nV := 1 to nVerts do
      LineTo(Trunc(xo + Vertices[nV].x), Trunc(yo + Vertices[nV].y));

    LineTo(Trunc(xo + Vertices[0].x), Trunc(yo + Vertices[0].y));
  end;
end;
{==============================================================================
                                 MoveIt
Purpose: To encapsalte the calls necessary to display and move an object
Details: You need to erase the object at the old location, then rotate the
         object, translte it to its new poisition and then redaw the object
==============================================================================}
procedure TGraphObject.MoveIt;
begin
  Draw(TRUE);
  Rotate;
  Translate;
  Draw(FALSE);
end;
{==============================================================================
                                 Rotate
Purpose: To spin the object around the z-axis
Details: Just take each vertice and based on the angle of the spin you can
         use mathematical equations to rotate each vertice
==============================================================================}
procedure TGraphObject.Rotate;
var
  nCntr: Integer;
  xNew : Real;
  yNew : Real;
begin

  for nCntr := 0 to nVerts do
  begin
    xNew := Vertices[nCntr].x * cs - Vertices[nCntr].y * sn;
    yNew := Vertices[nCntr].y * cs + Vertices[nCntr].x * sn;

    Vertices[nCntr].x := xNew;
    Vertices[nCntr].y := yNew;
  end;
end;
{==============================================================================
                                 SetColor
Purpose: Set the objects color
==============================================================================}
procedure TGraphObject.SetColor(Clr: TColor);
begin
  Color := Clr;
end;
{==============================================================================
                                 SetPos
Purpose: Set up the objects location
==============================================================================}
procedure TGraphObject.SetPos(xPos, yPos: Real);
begin
  xo := xPos;
  yo := yPos;
end;
{==============================================================================
                                 SetRotate
Purpose: Set the angle of rotation
Details: Precompute the cos and sin to help with speed. This is the only
         optimzation in this code.
==============================================================================}
procedure TGraphObject.SetRotate(NewAngle: Real);
begin
  Angle := NewAngle;
  cs    := cos(Angle);
  sn    := sin(Angle);
end;
{==============================================================================
                                 SetScale
Purpose: To determine the size of the object relative to its vertices
Details: Just take each vertice and either shink or expand it to get the new
         size of the object. Once you know the size of the object you need to
         calculate the bounding box that will enclose that object
==============================================================================}
procedure TGraphObject.SetScale(Scale: Real);
var
  nCntr: Integer;
begin
  rcBound := Rect(999, 999, -999, -999);

  for nCntr := 0 to nVerts do
  begin
    Vertices[nCntr].x := Vertices[nCntr].x * Scale;
    Vertices[nCntr].y := Vertices[nCntr].y * Scale;

    if (Vertices[nCntr].x < rcBound.Top) then
      rcBound.Top := Trunc(Vertices[nCntr].x);

    if (Vertices[nCntr].x > rcBound.Bottom) then
      rcBound.Bottom := Trunc(Vertices[nCntr].x);

    if (Vertices[nCntr].y < rcBound.Left) then
      rcBound.Left := Trunc(Vertices[nCntr].y);

    if (Vertices[nCntr].y > rcBound.Right) then
      rcBound.Right := Trunc(Vertices[nCntr].y);
  end;
end;
{==============================================================================
                                 SetVertices
Purpose: To set up the vertices that make up this object
==============================================================================}
procedure TGraphObject.SetVertices(Verts: Array of TVertex);
var
  nCntr : Integer;
begin
  if (nVerts > 14) then
    nVerts := 14
  else
    nVerts := High(Verts);

  for nCntr := 0 to nVerts do
    Vertices[nCntr] := Verts[nCntr];
end;
{==============================================================================
                                 SetVelocity
Purpose: To set the velocity of the object
Details: The velocity determines how fast the object will move in the x and
         y directions
==============================================================================}
procedure TGraphObject.SetVelocity(xV, yV: Real);
begin
  xVeloc := xV;
  yVeloc := yV;
end;
{==============================================================================
                                 Translate
Purpose: To move the object based on its velocities
Details: If the object has moved out of this worlds cooridnate space have it
         re-enter on the other side.
==============================================================================}
procedure TGraphObject.Translate;
begin
  xo := xo + xVeloc;
  yo := yo + yVeloc;

  if (xo > nW) then
    xo := Margins
  else if (xo < Margins) then
    xo := nW;

  if (yo > nH) then
    yo := Margins
  else if (yo < Margins) then
    yo := nH;
end;

{------------------------------------------------------------------------------
                          TDirectional Class Definitions
------------------------------------------------------------------------------}

{==============================================================================
                                 Create
Purpose: This function will create a TDirecional Object
Details: Set it so that it is not moving and it is facing North
==============================================================================}
constructor TDirectional.Create(Canv:TCanvas; Width, Height, MaxSpeed:Integer);
begin
  inherited Create(Canv, Width, Height);
  nMaxSpeed := MaxSpeed;
  Reset;
end;
{==============================================================================
                                 DecSpeed
Purpose: To Decrease the speed of the object
Details: If the speed is not already at its lowest then modify it and update
         the velocity
==============================================================================}
procedure TDirectional.DecSpeed;
begin
  if (nSpeed > 0) then
  begin
    Dec(nSpeed);
    UpdateVelocity;
  end;
end;
{==============================================================================
                                 IncSpeed
Purpose: To Increase the speed of the object
Details: If the speed is not already at its max then modify it and update
         the velocity
==============================================================================}
procedure TDirectional.IncSpeed;
begin
  if (nSpeed < nMaxSpeed) then
  begin
    Inc(nSpeed);
    UpdateVelocity;
  end;
end;
{==============================================================================
                                 MoveIt
Purpose: To Move a Directional Object The main difference is they do not rotate
Details: You need to erase the object at the old location, then translate it
         to its new poisition and then redaw the object
==============================================================================}
procedure TDirectional.MoveIt;
begin
  Draw(TRUE);
  Translate;
  Draw(FALSE);
end;
{==============================================================================
                                 Reset
Purpose: To initialize an object to a consitent starting state
==============================================================================}
procedure TDirectional.Reset;
begin
  nSpeed := 0;
  SetFacing(North); {Set so that we face 270 or to the Nouth}
end;
{==============================================================================
                                 SetFacing
Purpose: Sets the direction it is facing to 0 and then updates it
==============================================================================}
procedure TDirectional.SetFacing(F: Integer);
begin
  nFacing := 0;
  UpdateFacing(F);
end;
{==============================================================================
                                 SetSpeed
Purpose: Sets the speed the object is travling
Details: Validate the range and then update the velocity
==============================================================================}
procedure TDirectional.SetSpeed(S: Integer);
begin
  if (nSpeed < 0) then
    nSpeed := 0
  else if (nSpeed < nMaxSpeed) then
    nSpeed := S
  else
    nSpeed := S;
  UpdateVelocity;
end;
{==============================================================================
                                 UpdateFacing
Purpose: To Change the direction that the object faces
Details: The directions the ship can face are based on the number of degrees
         so you can face 0 .. NumDegrees. Once updated validate that you are
         facing in a valid direction and then convert the facing value into
         a radian so that the sin and cos can be calculated then update
         the velocity
==============================================================================}
procedure TDirectional.UpdateFacing(F: Integer);
var
  FM: Real;
begin
  Inc(nFacing, F);
  if (nFacing > NumDegrees) then
    nFacing := 0;
  if (nFacing < 0) then
    nFacing := NumDegrees;

  FM     := FaceMod * nFacing;
  csFace := cos(FM);
  snFace := sin(FM);
  UpdateVelocity;
end;
{==============================================================================
                                 UpdateVelocity
Purpose: This sets the velocity of the object based on the angle facing and
         the speed of the object
==============================================================================}
procedure TDirectional.UpdateVelocity;
begin
  SetVelocity(csFace * nSpeed, snFace * nSpeed);
end;
{------------------------------------------------------------------------------
                          TAsteroid Class Definitions
------------------------------------------------------------------------------}

{==============================================================================
                                 Create
Purpose: This function will create an Asteroid.
Details: Set up a random Color, then set up the vertices that will design
         the shape of the object, Randomly place the object in this world,
         set its velocity/direction, how it rotates, and its size.
==============================================================================}
constructor TAsteroid.Create(Canv: TCanvas; Width: Integer; Height: Integer);
begin
  inherited Create(Canv, Width, Height);
  Reset;
end;
{==============================================================================
                                 Destroy
Purpose: To destory an asteroid
Details: Flag it as destroyed, play the sound for hitting one, increment
         the score and then update status info and erase the asteroid.
         This would be the place to add splitting a large asteroid into
         two smaller asteroids.
==============================================================================}
procedure TAsteroid.Destroy;
begin
  bDestroyed := TRUE;
  Form1.PlaySound(sndHit);
  Inc(nScore, 100);
  Form1.UpdateCaption;
  Draw(TRUE);
end;
{==============================================================================
                                 MoveIt
Purpose: Move an object
Details: If the object has not been destroyed then have at it
==============================================================================}
procedure TAsteroid.MoveIt;
begin
  if bDestroyed = False then
    inherited MoveIt;
end;
{==============================================================================
                                 Reset
Purpose: To initialize an object to a consitent starting  state
==============================================================================}
procedure TAsteroid.Reset;
begin
  SetVertices([Vertex(4.0  + Random(2),  3.5 + Random(2)),
               Vertex(8.5  + Random(2), -3.0 + Random(2)),
               Vertex(6    + Random(2), -5   + Random(2)),
               Vertex(2    + Random(2), -3   + Random(2)),
               Vertex(-4   + Random(2), -6   + Random(2)),
               Vertex(-3.5 + Random(2),  5.5 + Random(2))]);

  SetPos(Margins + Random(nW), Margins + Random(nH));
  SetColor(Colors[Random(NumColors)]);
  SetVelocity(-10 + Random(20), -10 + Random(20));
  SetRotate((-50 + Random(100)) / 100);
  SetScale(Random(4) + 2);
  bDestroyed := False;
  Draw(False);
end;
{------------------------------------------------------------------------------
                          TMissle Class Definitions
------------------------------------------------------------------------------}

{==============================================================================
                                 Create
Purpose: Creates the missle
Details: Sets the objects shape, color, scale, speed, and deactivates it
==============================================================================}
constructor TMissle.Create(Canv:TCanvas; Width, Height, MaxSpeed: Integer);
begin
  inherited Create(Canv, Width, Height, MaxSpeed);
  SetVertices([Vertex(1.5,   0.5),  Vertex(0.5,   1.5),
               Vertex(-0.5,  1.5),  Vertex(-1.5,  0.5),
               Vertex(-1.5, -0.5),  Vertex(-0.5, -1.5),
               Vertex(0.5,  -1.5),  Vertex(1.5,  -0.5)]);
  SetColor(clWhite);
  SetScale(1);
  SetSpeed(0);
  nAlive := 0;
end;
{==============================================================================
                                 DestoryAsters
Purpose: To see if the missle has hit any asteroids
==============================================================================}
procedure TMissle.DestoryAsters;
var
  nCntr : Integer;
begin
  for nCntr := 0 to NumAsters do
  begin
    if ((Asteroids[nCntr].bDestroyed = False) and
        (Collide(Asteroids[nCntr]) = TRUE)) then
    begin
      Asteroids[nCntr].Destroy;
      nAlive := 0;
      Form1.UpdateCaption;
      break;
    end;
  end;
end;
{==============================================================================
                                 MoveIt
Purpose: If the object is to be drawn then draw it
Details: If it is a live missle then decrease the length of times it is
         alive and moveit.
==============================================================================}
procedure TMissle.MoveIt;
begin
  if (nAlive > 0) then
  begin
    Dec(nAlive);
    inherited MoveIt;
    DestoryAsters;
    if (nAlive = 0) then  { Erase it at the end of its life }
      Draw(TRUE);
  end;
end;
{==============================================================================
                                 SetAlive
Purpose: Sets the duration that the object will be drawn
==============================================================================}
procedure TMissle.SetAlive(nA: Integer);
begin
  nAlive := nA;
end;
{------------------------------------------------------------------------------
                          TShip Class Definitions
------------------------------------------------------------------------------}

{==============================================================================
                                 Create
Purpose: Create a ship
Details: Sets the ships shape, position it in the middle, sets it color,
         sets it to not moving and not firing, then it creates a Missle
==============================================================================}
constructor TShip.Create(Canv:TCanvas; Width, Height, MaxSpeed: Integer);
var
  nCntr : Integer;
begin
  inherited Create(Canv, Width, Height, MaxSpeed);
  for nCntr := 0 to NumMissles do
    Missle[nCntr] := TMissle.Create(Canv, Width, Height, 30);
  Reset;
end;
{==============================================================================
                                 Free
Purpose: To release the resources that a ship uses
==============================================================================}
procedure TShip.Free;
var
  nCntr : Integer;
begin
  for nCntr := 0 to NumMissles do
    Missle[nCntr].Free;
  inherited Free;
end;
{==============================================================================
                                 BlowUp
Purpose: To destory the ship when it has been hit
==============================================================================}
procedure TShip.BlowUp;
var
  nCntr: Integer;
  nxo  : Integer;
  nyo  : Integer;
  nxr  : Integer;
  nyr  : Integer;
begin
  Form1.Timer1.Enabled := False;
  Form1.Panel1.Caption := 'You have been Hit';
  Form1.PlaySound(sndExplode);

  nxr := rcBound.Bottom - rcBound.Top  + Trunc(Margins / 2);
  nyr := rcBound.Right  - rcBound.Left + Trunc(Margins / 2);
  nxo := Trunc(xo - nxr / 2);
  nyo := Trunc(yo - nyr / 2);

  for nCntr := 0 To 2500 do
    Canvas.Pixels[nxo + Random(nxr),nyo + Random(nyr)] :=
        Colors[Random(NumColors)];
end;
{==============================================================================
                                 MoveIt
Purpose: To move the ship and see if has been hit
Details: If the ship needs to be rotated then it only happens once and
         Then the missles are drawn
==============================================================================}
procedure TShip.MoveIt;
var
  nCntr : Integer;
begin
  Draw(TRUE);
  if bRotate = TRUE then
  begin
    Rotate;
    bRotate := FALSE;
  end;
  Translate;
  Draw(FALSE);
  for nCntr := 0 to NumMissles do
    Missle[nCntr].MoveIt;
end;
{==============================================================================
                                 RotateRight
Purpose: To rotate the ship to the right
Details: Set the angle of rotation, modify the direction it is faceing,
         set the flag to rotate next time it is moved
==============================================================================}
procedure TShip.RotateRight;
begin
  SetRotate(FaceMod);
  UpdateFacing(1);
  bRotate := TRUE;
end;
{==============================================================================
                                 Reset
Purpose: To initialize an object to a consitent starting state
==============================================================================}
procedure TShip.Reset;
var
  nCntr: Integer;
begin
  inherited Reset;
  SetVertices([Vertex(3,-19),   Vertex(12, -1),
               Vertex(17, 2),   Vertex(17, 9),
               Vertex(8, 14),   Vertex(5, 8),
               Vertex(-5, 8),   Vertex(-8, 14),
               Vertex(-17, 9),  Vertex(-17, 2),
               Vertex(-12, -1), Vertex(-3, -19),
               Vertex(-3, -8),  Vertex(3, -8)]);
  SetPos(nW DIV 2, nH DIV 2);
  SetColor(clYellow);
  SetScale(1);
  for nCntr := 0 to NumMissles do
    Missle[nCntr].SetAlive(0);
  Draw(False);
end;
{==============================================================================
                                 RotateLeft
Purpose: To rotate the ship to the left
Details: Set the angle of rotation, modify the direction it is faceing,
         set the flag to rotate next time it is moved
==============================================================================}
procedure TShip.RotateLeft;
begin
  SetRotate(-FaceMod);
  UpdateFacing(-1);
  bRotate := TRUE;
end;
{==============================================================================
                                 Shoot
Purpose: To fire a missle
Details: If the missle is not alive then skip it
==============================================================================}
procedure TShip.Shoot;
var
  nCntr: Integer;
begin
  for nCntr := 0 to NumMissles do
  begin
    if (Missle[nCntr].nAlive = 0) then
    begin
      Form1.PlaySound(sndFire);
      Missle[nCntr].SetPos(xo, yo);
      Missle[nCntr].SetFacing(nFacing);
      Missle[nCntr].SetSpeed(Missle[nCntr].nMaxSpeed);
      Missle[nCntr].SetAlive(Random(10) + 10);
    end;
  end;
end;
{------------------------------------------------------------------------------
                          Delphi UI Class Definitions
------------------------------------------------------------------------------}

{==============================================================================
                                 BeginGame
Purpose: To start a new game
==============================================================================}
procedure TForm1.BeginGame;
var
  nCntr: Integer;
begin
  Canvas.Pen.Color := clBlack;
  Canvas.Rectangle(0, 0, ClientWidth, ClientHeight);
  nScore         := 0;
  for nCntr  := 0 to NumAsters do
    Asteroids[nCntr].Reset;
  Ship.Reset;
  UpdateCaption;
  Panel1.Caption := 'Use Arrow Keys to move and Space bar to Fire';
  Timer1.Enabled := TRUE;
end;
{==============================================================================
                                 FormCreate
Purpose: To setup the events necessary when the form is created
Details: Determine how many degrees, where north is, seed the random
         number generator, set up the colors, get ini file info,
         and create the objects
==============================================================================}
procedure TForm1.FormCreate(Sender: TObject);
var
 nCntr      : Integer;
 NumDraws   : LongInt;
 NumLast    : LongInt;
 iniFile    : TIniFile;
 strSec     : String;
 strDrawSec : String;
 nTurnDeg   : Integer;
 w          : WORD;
 IniPath    : String;

begin
  nTurnDeg   := 20;                  { Resolution that the ship can turn }
  FaceMod    := nTurnDeg * PI / 180; { This many Radians in nTurnDeg Degrees }
  NumDegrees := Trunc(360 / nTurnDeg) - 1;
  North      := Trunc((NumDegrees + 1) * 0.75);
{
               If I used 45 Degree turning Resolution you would have
  FaceMod    = 0.7853981634
  NumDegrees = 7

          Valid Facings would run From 0 to 7 and would represent

     Face  -- Degree  -- Direction Ship Moves
     0:         0           East
     1:         45          South East
     2:         90          South
     3:         135         South West
     4:         180         West
     5:         225         North West
     6:         270         North
     7:         315         North East

  We start facing North so it will be the (NumDegrees + 1 * 3/4)
}
  Randomize;
  Colors[0]  := clAqua;
  Colors[1]  := clBlue;
  Colors[2]  := clFuchsia;
  Colors[3]  := clGreen;
  Colors[4]  := clLime;
  Colors[5]  := clGray;
  Colors[6]  := clOlive;
  Colors[7]  := clPurple;
  Colors[8]  := clRed;
  Colors[9]  := clSilver;
  Colors[10] := clTeal;
  Colors[11] := clWhite;
  Colors[12] := clYellow;
  TimesDrawn := 0;
  Left       := 0;
  Top        := 0;
  Color      := ClBlack;
  Width      := GetSystemMetrics(SM_CXSCREEN);
  Height     := GetSystemMetrics(SM_CYSCREEN);
  IniPath    := ExtractFilePath(Application.ExeName) + 'asteroid.ini';
  iniFile    := TiniFile.Create(IniPath);
  NumAsters  := iniFile.ReadInteger('Settings', 'Asteroids', 10);
  NumDraws   := iniFile.ReadInteger('Timing', 'Draws',     0);
  strSec     := iniFile.ReadString('Timing',  'Seconds',  '0.00');
  strDrawSec := iniFile.ReadString('Timing',  'Draws_Sec','0.00' );
  iniFile.Free;
  Dec(NumAsters);
  
  if (NumAsters > MaxAsters) then
    NumAsters := MaxAsters;
    
  for nCntr  := 0 to NumAsters do
    Asteroids[nCntr] := TAsteroid.Create(Canvas,
                        ClientWidth - Margins * 2, ClientHeight - Margins * 2);

  Ship := TShip.Create(Canvas,ClientWidth - Margins * 2,
                       ClientHeight - Margins * 2, 20);
                       
  SndPlayer.FileName := 'AsterSnd.wav';
  SndPlayer.Open;
  StartTicks := GetTickCount;
  BeginGame;
end;
{==============================================================================
                                 Timer1Timer
Purpose: To perform one game cycle every tick about 18.2 times/sec
==============================================================================}
procedure TForm1.Timer1Timer(Sender: TObject);
var
 nCntr : Integer;
 nAlive: Integer;
begin
  for nCntr := 0 to NumAsters do
  begin
    Asteroids[nCntr].MoveIt;
    TimesDrawn := TimesDrawn + 1;
  end;
  Ship.MoveIt;
  TimesDrawn := TimesDrawn + 1;

  nAlive := 0;
  for nCntr := 0 to NumAsters do
  begin
    if (Asteroids[nCntr].bDestroyed = False) then
    begin
      Inc(nAlive);
      if (Ship.Collide(Asteroids[nCntr]) = TRUE) then
        Ship.BlowUp;
    end;
  end;

  if (nAlive = 0) then
  begin
    Timer1.Enabled := False;
    Form1.Panel1.Caption := 'You Have Won';
    Form1.PlaySound(sndWin);
  end;
end;
{==============================================================================
                                 FormClose
Purpose: Shut down the application
==============================================================================}
procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
var
  iniFile  : TIniFile;
  iniPath  : String;
  Seconds  : Real;
  DrawsSec : Real;
  strVal   : String;
  nCntr    : Integer;
begin
  Seconds  := (GetTickCount - StartTicks) / 1000;
  DrawsSec := TimesDrawn / Seconds;

  SndPlayer.Close;
  Ship.Free;
  for nCntr := 0 to NumAsters do
    Asteroids[nCntr].Free;

  IniPath := ExtractFilePath(Application.ExeName) + 'asteroid.ini';
  iniFile := TiniFile.Create(IniPath);
  iniFile.WriteInteger('Settings', 'Asteroids', NumAsters + 1);
  iniFile.WriteInteger('Timing', 'Draws',     TimesDrawn);
  FmtStr(strVal, '%4.4f', [Seconds]);
  iniFile.WriteString('Timing', 'Seconds',   strVal);
  FmtStr(strVal, '%4.4f', [DrawsSec]);
  iniFile.WriteString('Timing', 'Draws_Sec',   strVal);
  iniFile.Free;
  Action := caFree;
end;
{==============================================================================
                                 clsButtonClick
Purpose: Close the application
==============================================================================}
procedure TForm1.clsButtonClick(Sender: TObject);
begin
  Close;
end;
{==============================================================================
                                 btnPauseClick
Purpose: Pause the application and display the about box
==============================================================================}
procedure TForm1.btnPauseClick(Sender: TObject);
begin
  Timer1.Enabled := False;
  frmAbout.ShowModal;
  Timer1.Enabled := True;
end;
{==============================================================================
                                 FormKeyUp
Purpose: To respond to keyboard events and have the ship react
==============================================================================}
procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    VK_UP:
      begin
      Ship.IncSpeed;
      Key := 0;
      end;

    VK_DOWN:
      begin
      Ship.DecSpeed;
      Key := 0;
      end;

    VK_LEFT:
      begin
      Ship.RotateLeft;
      Key := 0;
      end;

    VK_RIGHT:
      begin
      Ship.RotateRight;
      Key := 0;
      end;

    VK_SPACE:
      begin
      Ship.Shoot;
      Key := 0;
      end;
  end;
end;
{==============================================================================
                                 btnReplayClick
Purpose: To start the game over.
==============================================================================}
procedure TForm1.btnReplayClick(Sender: TObject);
begin
  BeginGame;
end;
{==============================================================================
                                 PlaySound
Purpose: To play the sound requested
Details: Notice how the media player loads the wav file once, this gives us
         much better performance to play the sound. So all sounds are in one
         file and the offsets are set so that the sounds play quickly. Also
         the whole sound file is only about 4 seconds which windows wav buffer
         defaults to.
==============================================================================}
procedure TForm1.PlaySound(nSound: Integer);
begin
  case nSound of
    SndFire:
    begin
      SndPlayer.StartPos := 0;
      SndPlayer.EndPos   := 510;
    end;

    SndHit:
    begin
      SndPlayer.StartPos := 511;
      SndPlayer.EndPos   := 1130;
    end;

    SndExplode:
    begin
      SndPlayer.StartPos := 1140;
      SndPlayer.EndPos   := 2580;
    end;

    SndWin:
    begin
      SndPlayer.StartPos := 2581;
      SndPlayer.EndPos   := 4650;
    end;
  end;

  SndPlayer.Play;
end;
{==============================================================================
                                 UpdateCaption
Purpose: Show the user their status information
==============================================================================}
procedure TForm1.UpdateCaption;
var
  nCntr : Integer;
  nAlive: Integer;
begin

  nAlive := 0;
  for nCntr := 0 to NumAsters do
  begin
    if (Asteroids[nCntr].bDestroyed = False) then
      Inc(nAlive);
  end;

  Panel1.Caption := Format('Asteroids: %d    Number Left: %d',
                            [NumAsters + 1, nAlive]);
  Panel3.Caption := 'Score: ' + IntToStr(nScore);
end;
end.
